@@ -4,7 +4,7 @@ import re
 from rich.console import Console, RenderableType
 
 
-re_link_ids = re.compile(r"id=[\d\.\-]*?;.*?\x1b")
+re_link_ids = re.compile(r"id=[\d.\-]*?;.*?\x1b")
 
 
 def replace_link_ids(render: str) -> str:
