@@ -1,7 +1,8 @@
 import pytest
 
 from rich.console import Console
-from rich.markup import escape, MarkupError, _parse, render, Tag, RE_TAGS
+from rich.errors import MarkupError
+from rich.markup import RE_TAGS, Tag, _parse, escape, render
 from rich.text import Span
 
 
@@ -39,6 +40,9 @@ def test_escape():
     assert escape("[@foo]") == "\\[@foo]"
     assert escape("[@]") == "\\[@]"
 
+    # https://github.com/Textualize/rich/issues/2187
+    assert escape("[nil, [nil]]") == r"[nil, \[nil]]"
+
 
 def test_render_escape():
     console = Console(width=80, color_system=None)
@@ -139,6 +143,11 @@ def test_markup_error():
         assert render("[foo]hello[/bar]")
 
 
+def test_markup_escape():
+    result = str(render("[dim white][url=[/]"))
+    assert result == "[url="
+
+
 def test_escape_escape():
     # Escaped escapes (i.e. double backslash)should be treated as literal
     result = render(r"\\[bold]FOO")
@@ -165,7 +174,6 @@ def test_escape_escape():
 
 
 def test_events():
-
     result = render("[@click]Hello[/@click] [@click='view.toggle', 'left']World[/]")
     assert str(result) == "Hello World"
 
