@@ -1,8 +1,8 @@
 from time import sleep
 
 from rich.console import Console
+from rich.spinner import Spinner
 from rich.status import Status
-from rich.table import Table
 
 
 def test_status():
@@ -12,9 +12,13 @@ def test_status():
     )
     status = Status("foo", console=console)
     assert status.console == console
-    status.update(status="bar", spinner="dots2", spinner_style="red", speed=2.0)
+    previous_status_renderable = status.renderable
+    status.update(status="bar", spinner_style="red", speed=2.0)
 
-    assert isinstance(status.renderable, Table)
+    assert previous_status_renderable == status.renderable
+    assert isinstance(status.renderable, Spinner)
+    status.update(spinner="dots2")
+    assert previous_status_renderable != status.renderable
 
     # TODO: Testing output is tricky with threads
     with status:
